@@ -3,7 +3,7 @@ use warnings;
 
 package HTTP::Async;
 
-our $VERSION = '0.10';
+our $VERSION = '0.23';
 
 use Carp;
 use Data::Dumper;
@@ -81,13 +81,16 @@ C<select> lists.
 There are a number of default settings that should be suitable for most uses.
 However in some circumstances you might wish to change these.
 
-            slots: 20
-          timeout: 180 (seconds)
- max_request_time: 300 (seconds)
-    max_redirects: 7
-    poll_interval: 0.05 (seconds)
-       proxy_host: ''
-       proxy_port: ''
+            slots:  20
+          timeout:  180 (seconds)
+ max_request_time:  300 (seconds)
+    max_redirects:  7
+    poll_interval:  0.05 (seconds)
+       proxy_host:  ''
+       proxy_port:  ''
+       local_addr:  ''
+       local_port:  ''
+       ssl_options: {}
        
 =head1 METHODS
 
@@ -136,7 +139,7 @@ sub _init {
 
 sub _next_id { return ++$_[0]->{current_id} }
 
-=head2 slots, timeout, max_request_time, poll_interval, max_redirects, proxy_host and proxy_port
+=head2 slots, timeout, max_request_time, poll_interval, max_redirects, proxy_host, proxy_port, local_addr, local_port, ssl_options
 
     $old_value = $async->slots;
     $new_value = $async->slots( $new_value );
@@ -150,7 +153,7 @@ Slots is the maximum number of parallel requests to make.
 
 my %GET_SET_KEYS = map { $_ => 1 } qw( slots poll_interval
   timeout max_request_time max_redirects
-  proxy_host proxy_port );
+  proxy_host proxy_port local_addr local_port ssl_options);
 
 sub _add_get_set_key {
     my $class = shift;
@@ -243,7 +246,7 @@ sub add_with_opts {
     $async->poke;
 
 At fairly frequent intervals some housekeeping needs to performed - such as
-reading recieved data and starting new requests. Calling C<poke> lets the
+reading received data and starting new requests. Calling C<poke> lets the
 object do this and then return quickly. Usually you will not need to use this
 as most other methods do it for you.
 
@@ -639,6 +642,8 @@ sub _send_request {
     $args{Host}     = $uri->host;
     $args{PeerAddr} = $self->_get_opt( 'proxy_host', $id );
     $args{PeerPort} = $self->_get_opt( 'proxy_port', $id );
+    $args{LocalAddr} = $self->_get_opt('local_addr', $id );
+    $args{LocalPort} = $self->_get_opt('local_port', $id );
 
     my $request_is_to_proxy =
       ( $args{PeerAddr} || $args{PeerPort} )    # if either are set...
@@ -650,13 +655,26 @@ sub _send_request {
     $args{PeerPort} ||= $uri->port;
 
     my $net_http_class = 'Net::HTTP::NB';
-    if ($uri->scheme and $uri->scheme eq 'https') {
+    if ($uri->scheme and $uri->scheme eq 'https' and not $request_is_to_proxy) {
         $net_http_class = 'Net::HTTPS::NB';
         eval {
             require Net::HTTPS::NB;
             Net::HTTPS::NB->import();
         };
         die "$net_http_class must be installed for https support" if $@;
+
+        # Add SSL options, if any, to args
+        my $ssl_options = $self->_get_opt('ssl_options');
+        @args{ keys %$ssl_options } = values %$ssl_options if $ssl_options;
+    }
+    elsif($uri->scheme and $uri->scheme eq 'https' and $request_is_to_proxy) {
+        # We are making an HTTPS request through an HTTP proxy such as squid.
+        # The proxy will handle the HTTPS, we need to connect to it via HTTP
+        # and then make a request where the https is clear from the scheme...
+        $args{Host} = sprintf(
+            '%s:%s',
+            delete @args{'PeerAddr', 'PeerPort'}
+        );
     }
     my $s = eval { $net_http_class->new(%args) };
 
@@ -675,7 +693,10 @@ sub _send_request {
         return 1;
     }
 
-    my %headers = %{ $request->{_headers} };
+    my %headers;
+    for my $key ($request->{_headers}->header_field_names) {
+        $headers{$key} = $request->header($key);
+    }
 
     # Decide what to use as the request_uri
     my $request_uri = $request_is_to_proxy    # is this a proxy request....
@@ -732,16 +753,7 @@ sub _make_url_absolute {
     my $in  = $args{url};
     my $ref = $args{ref};
 
-    return $in if $in =~ m{ \A http:// }xms;
-
-    my $ret = $ref->scheme . '://' . $ref->authority;
-    return $ret . $in if $in =~ m{ \A / }xms;
-
-    $ret .= $ref->path;
-    return $ret . $in if $in =~ m{ \A [\?\#\;] }xms;
-
-    $ret =~ s{ [^/]+ \z }{}xms;
-    return $ret . $in;
+    return URI->new_abs($in, $ref)->as_string;
 }
 
 sub _add_error_response_to_return {
@@ -784,6 +796,18 @@ handle 304 responses.
 
 Naveed Massjouni for adding the https handling code.
 
+Alex Balhatchet for adding the https + proxy handling code, and for making the
+tests run ok in parallel.
+
+Josef Toman for fixing two bugs, one related to header handling and another
+related to producing an absolute URL correctly.
+
+Github user 'c00ler-' for adding LocalAddr and LocalPort support.
+
+rt.cpan.org user 'Florian (fschlich)' for typo in documentation.
+
+Heikki Vatiainen for the ssl_options support patch.
+
 =head1 BUGS AND REPO
 
 Please submit all bugs, patches etc on github
